@@ -29,6 +29,7 @@
 #include "dir.h"
 #include "doxygen.h"
 #include "outputgen.h"
+#include "outputlist.h"
 #include "parserintf.h"
 #include "classdef.h"
 #include "namespacedef.h"
@@ -39,7 +40,7 @@
 #include "filename.h"
 #include "version.h"
 
-class XRefDummyCodeGenerator : public CodeOutputInterface
+class XRefDummyCodeGenerator : public OutputCodeExtension
 {
   public:
     XRefDummyCodeGenerator(FileDef *fd) : m_fd(fd) {}
@@ -47,7 +48,7 @@ class XRefDummyCodeGenerator : public CodeOutputInterface
 
     // these are just null functions, they can be used to produce a syntax highlighted
     // and cross-linked version of the source code, but who needs that anyway ;-)
-    OutputType type() const override { return OutputType::Null; }
+    OutputType type() const override { return OutputType::Extension; }
     void codify(const QCString &) override {}
     void writeCodeLink(CodeSymbolType,const QCString &,const QCString &,const QCString &,const QCString &,const QCString &) override  {}
     void writeLineNumber(const QCString &,const QCString &,const QCString &,int,bool) override {}
@@ -59,8 +60,6 @@ class XRefDummyCodeGenerator : public CodeOutputInterface
     void startFontClass(const QCString &) override {}
     void endFontClass() override {}
     void writeCodeAnchor(const QCString &) override {}
-    void setCurrentDoc(const Definition *,const QCString &,bool) override {}
-    void addWord(const QCString &,bool) override {}
     void startCodeFragment(const QCString &) override {}
     void endCodeFragment(const QCString &) override {}
 
@@ -119,19 +118,18 @@ static void findXRefSymbols(FileDef *fd)
   intf->resetCodeParserState();
 
   // create a new backend object
-  XRefDummyCodeGenerator *xrefGen = new XRefDummyCodeGenerator(fd);
+  XRefDummyCodeGenerator xrefGen(fd);
+  OutputCodeList xrefList;
+  xrefList.add(OutputCodeDeferExtension(&xrefGen));
 
   // parse the source code
-  intf->parseCode(*xrefGen,
+  intf->parseCode(xrefList,
                 0,
                 fileToString(fd->absFilePath()),
                 lang,
                 FALSE,
                 0,
                 fd);
-
-  // dismiss the object.
-  delete xrefGen;
 }
 
 static void listSymbol(Definition *d)
@@ -304,7 +302,7 @@ int main(int argc,char **argv)
   while (1)
   {
     printf("> Type a symbol name or\n> .list for a list of symbols or\n> .quit to exit\n> ");
-    fgets(cmd,256,stdin);
+    (void)fgets(cmd,256,stdin);
     QCString s(cmd);
     if (s.at(s.length()-1)=='\n') s=s.left(s.length()-1); // strip trailing \n
     if (s==".list")
