@@ -50,6 +50,8 @@
 #include "dirdef.h"
 #include "section.h"
 #include "dir.h"
+#include "growbuf.h"
+#include "outputlist.h"
 
 // no debug info
 #define Docbook_DB(x) do {} while(0)
@@ -102,7 +104,7 @@ inline void writeDocbookCodeString(TextStream &t,const QCString &str, int &col)
       case '"':  t << "&quot;"; col++; break;
       default:
         {
-          uchar uc = static_cast<uchar>(c);
+          uint8_t uc = static_cast<uint8_t>(c);
           static const char *hex="0123456789ABCDEF";
           if (uc<32)
           {
@@ -143,14 +145,14 @@ void writeDocbookLink(TextStream &t,const QCString & /*extRef*/,const QCString &
   t << "</link>";
 }
 
-DocbookCodeGenerator::DocbookCodeGenerator(TextStream &t) : m_t(t)
+DocbookCodeGenerator::DocbookCodeGenerator(TextStream *t) : m_t(t)
 {
 }
 
 void DocbookCodeGenerator::codify(const QCString &text)
 {
   Docbook_DB(("(codify \"%s\")\n",text));
-  writeDocbookCodeString(m_t,text,m_col);
+  writeDocbookCodeString(*m_t,text,m_col);
 }
 
 void DocbookCodeGenerator::writeCodeLink(CodeSymbolType,
@@ -159,7 +161,7 @@ void DocbookCodeGenerator::writeCodeLink(CodeSymbolType,
     const QCString &tooltip)
 {
   Docbook_DB(("(writeCodeLink)\n"));
-  writeDocbookLink(m_t,ref,file,anchor,name,tooltip);
+  writeDocbookLink(*m_t,ref,file,anchor,name,tooltip);
   m_col+=name.length();
 }
 
@@ -170,10 +172,10 @@ void DocbookCodeGenerator::writeCodeLinkLine(CodeSymbolType,
 {
   Docbook_DB(("(writeCodeLinkLine)\n"));
   if (!writeLineAnchor) return;
-  m_t << "<anchor xml:id=\"_" << stripExtensionGeneral(stripPath(file),".xml");
-  m_t << "_1l";
-  writeDocbookString(m_t,name);
-  m_t << "\"/>";
+  *m_t << "<anchor xml:id=\"_" << stripExtensionGeneral(stripPath(file),".xml");
+  *m_t << "_1l";
+  writeDocbookString(*m_t,name);
+  *m_t << "\"/>";
   m_col+=name.length();
 }
 
@@ -193,7 +195,7 @@ void DocbookCodeGenerator::startCodeLine(bool)
 
 void DocbookCodeGenerator::endCodeLine()
 {
-  if (m_insideCodeLine) m_t << "\n";
+  if (m_insideCodeLine) *m_t << "\n";
   Docbook_DB(("(endCodeLine)\n"));
   m_lineNumber = -1;
   m_refId.resize(0);
@@ -204,14 +206,14 @@ void DocbookCodeGenerator::endCodeLine()
 void DocbookCodeGenerator::startFontClass(const QCString &colorClass)
 {
   Docbook_DB(("(startFontClass)\n"));
-  m_t << "<emphasis role=\"" << colorClass << "\">";
+  *m_t << "<emphasis role=\"" << colorClass << "\">";
   m_insideSpecialHL=TRUE;
 }
 
 void DocbookCodeGenerator::endFontClass()
 {
   Docbook_DB(("(endFontClass)\n"));
-  m_t << "</emphasis>"; // non DocBook
+  *m_t << "</emphasis>"; // non DocBook
   m_insideSpecialHL=FALSE;
 }
 
@@ -242,11 +244,11 @@ void DocbookCodeGenerator::writeLineNumber(const QCString &ref,const QCString &f
     {
       codify(lineNumber);
     }
-    m_t << " ";
+    *m_t << " ";
   }
   else
   {
-    m_t << l << " ";
+    *m_t << l << " ";
   }
   m_col=0;
 }
@@ -258,44 +260,93 @@ void DocbookCodeGenerator::finish()
 
 void DocbookCodeGenerator::startCodeFragment(const QCString &)
 {
-DB_GEN_C1(m_t)
-  m_t << "<programlisting linenumbering=\"unnumbered\">";
+DB_GEN_C1(*m_t)
+  *m_t << "<programlisting linenumbering=\"unnumbered\">";
 }
 
 void DocbookCodeGenerator::endCodeFragment(const QCString &)
 {
-DB_GEN_C1(m_t)
+DB_GEN_C1(*m_t)
   //endCodeLine checks is there is still an open code line, if so closes it.
   endCodeLine();
 
-  m_t << "</programlisting>";
+  *m_t << "</programlisting>";
 }
 
 //-------------------------------------------------------------------------------
 
-DocbookGenerator::DocbookGenerator() : OutputGenerator(Config_getString(DOCBOOK_OUTPUT)), m_codeGen(m_t)
+DocbookGenerator::DocbookGenerator()
+  : OutputGenerator(Config_getString(DOCBOOK_OUTPUT))
+  , m_codeList(std::make_unique<OutputCodeList>())
 {
 DB_GEN_C
+  m_codeGen = m_codeList->add<DocbookCodeGenerator>(&m_t);
 }
 
-DocbookGenerator::DocbookGenerator(const DocbookGenerator &og) : OutputGenerator(og), m_codeGen(m_t)
+DocbookGenerator::DocbookGenerator(const DocbookGenerator &og) : OutputGenerator(og.m_dir)
 {
+  m_codeList         = std::make_unique<OutputCodeList>(*og.m_codeList);
+  m_codeGen          = m_codeList->get<DocbookCodeGenerator>();
+  m_codeGen->setTextStream(&m_t);
+  m_denseText        = og.m_denseText;
+  m_inGroup          = og.m_inGroup;
+  m_levelListItem    = og.m_levelListItem;
+  m_inListItem       = og.m_inListItem;
+  m_inSimpleSect     = og.m_inSimpleSect;
+  m_descTable        = og.m_descTable;
+  m_simpleTable      = og.m_simpleTable;
+  m_inLevel          = og.m_inLevel;
+  m_firstMember      = og.m_firstMember;
+  m_openSectionCount = og.m_openSectionCount;
 }
 
 DocbookGenerator &DocbookGenerator::operator=(const DocbookGenerator &og)
 {
-  OutputGenerator::operator=(og);
+  if (this!=&og)
+  {
+    m_dir          = og.m_dir;
+    m_codeList     = std::make_unique<OutputCodeList>(*og.m_codeList);
+    m_codeGen      = m_codeList->get<DocbookCodeGenerator>();
+    m_codeGen->setTextStream(&m_t);
+    m_denseText        = og.m_denseText;
+    m_inGroup          = og.m_inGroup;
+    m_levelListItem    = og.m_levelListItem;
+    m_inListItem       = og.m_inListItem;
+    m_inSimpleSect     = og.m_inSimpleSect;
+    m_descTable        = og.m_descTable;
+    m_simpleTable      = og.m_simpleTable;
+    m_inLevel          = og.m_inLevel;
+    m_firstMember      = og.m_firstMember;
+    m_openSectionCount = og.m_openSectionCount;
+  }
   return *this;
 }
 
-std::unique_ptr<OutputGenerator> DocbookGenerator::clone() const
+DocbookGenerator::DocbookGenerator(DocbookGenerator &&og)
+  : OutputGenerator(std::move(og))
 {
-  return std::make_unique<DocbookGenerator>(*this);
+  m_codeList         = std::exchange(og.m_codeList,std::unique_ptr<OutputCodeList>());
+  m_codeGen          = m_codeList->get<DocbookCodeGenerator>();
+  m_codeGen->setTextStream(&m_t);
+  m_denseText        = std::exchange(og.m_denseText,false);
+  m_inGroup          = std::exchange(og.m_inGroup,false);
+  m_levelListItem    = std::exchange(og.m_levelListItem,0);
+  m_inListItem       = std::exchange(og.m_inListItem,std::array<bool,20>());
+  m_inSimpleSect     = std::exchange(og.m_inSimpleSect,std::array<bool,20>());
+  m_descTable        = std::exchange(og.m_descTable,false);
+  m_simpleTable      = std::exchange(og.m_simpleTable,false);
+  m_inLevel          = std::exchange(og.m_inLevel,-1);
+  m_firstMember      = std::exchange(og.m_firstMember,false);
+  m_openSectionCount = std::exchange(og.m_openSectionCount,0);
 }
 
 DocbookGenerator::~DocbookGenerator()
 {
-DB_GEN_C
+}
+
+void DocbookGenerator::addCodeGen(OutputCodeList &list)
+{
+  list.add(OutputCodeList::OutputCodeVariant(DocbookCodeGeneratorDefer(m_codeGen)));
 }
 
 void DocbookGenerator::init()
@@ -317,7 +368,7 @@ void DocbookGenerator::cleanup()
 }
 
 
-void DocbookGenerator::startFile(const QCString &name,const QCString &,const QCString &,int)
+void DocbookGenerator::startFile(const QCString &name,const QCString &,const QCString &,int,int)
 {
 DB_GEN_C
   QCString fileName=name;
@@ -337,8 +388,9 @@ DB_GEN_C
   relPath = relativePathToRoot(fileName);
   if (!fileName.endsWith(".xml")) fileName+=".xml";
   startPlainFile(fileName);
-  m_codeGen.setRelativePath(relPath);
-  m_codeGen.setSourceFileName(stripPath(fileName));
+  m_codeGen->setRelativePath(relPath);
+  m_codeGen->setSourceFileName(stripPath(fileName));
+  m_pageLinks = QCString();
 
   m_t << "<?xml version='1.0' encoding='UTF-8' standalone='no'?>\n";;
   m_t << "<" << fileType << " xmlns=\"http://docbook.org/ns/docbook\" version=\"5.0\" xmlns:xlink=\"http://www.w3.org/1999/xlink\"";
@@ -354,8 +406,11 @@ DB_GEN_C
   m_inLevel = -1;
   m_inGroup = FALSE;
 
+  // Write page links only after all sections have been closed to avoid bugs
+  m_t << m_pageLinks;
+
   QCString fileType="section";
-  QCString fileName= m_codeGen.sourceFileName();
+  QCString fileName= m_codeGen->sourceFileName();
   if (fileName == "index.xml")
   {
     fileType="book";
@@ -366,15 +421,15 @@ DB_GEN_C
   }
   m_t << "</" << fileType << ">\n";
   endPlainFile();
-  m_codeGen.setSourceFileName("");
+  m_codeGen->setSourceFileName("");
 }
 
-void DocbookGenerator::startIndexSection(IndexSections is)
+void DocbookGenerator::startIndexSection(IndexSection is)
 {
-DB_GEN_C2("IndexSections " << is)
+DB_GEN_C2("IndexSection " << is)
   switch (is)
   {
-    case isTitlePageStart:
+    case IndexSection::isTitlePageStart:
       {
         QCString dbk_projectName = Config_getString(PROJECT_NAME);
         m_t << "    <info>\n";
@@ -382,131 +437,132 @@ DB_GEN_C2("IndexSections " << is)
         m_t << "    </info>\n";
       }
       break;
-    case isTitlePageAuthor:
+    case IndexSection::isTitlePageAuthor:
       break;
-    case isMainPage:
-      m_t << "<chapter>\n";
-      m_t << "    <title>";
+    case IndexSection::isMainPage:
       break;
-    case isModuleIndex:
-      //Module Index}\n"
+    case IndexSection::isModuleIndex:
+      //Module Index\n"
       break;
-    case isDirIndex:
-      //Directory Index}\n"
+    case IndexSection::isDirIndex:
+      //Directory Index\n"
       break;
-    case isNamespaceIndex:
-      //Namespace Index}\n"
+    case IndexSection::isNamespaceIndex:
+      //Namespace Index\n"
       break;
-    case isConceptIndex:
-      //Concept Index}\n"
+    case IndexSection::isConceptIndex:
+      //Concept Index\n"
       break;
-    case isClassHierarchyIndex:
-      //Hierarchical Index}\n"
+    case IndexSection::isClassHierarchyIndex:
+      //Hierarchical Index\n"
       break;
-    case isCompoundIndex:
+    case IndexSection::isCompoundIndex:
       //m_t << "{"; //Class Index}\n"
       break;
-    case isFileIndex:
-      //Annotated File Index}\n"
+    case IndexSection::isFileIndex:
+      //Annotated File Index\n"
       break;
-    case isPageIndex:
-      //Annotated Page Index}\n"
+    case IndexSection::isPageIndex:
+      //Annotated Page Index\n"
       break;
-    case isModuleDocumentation:
+    case IndexSection::isModuleDocumentation:
       m_t << "<chapter>\n";
       m_t << "    <title>";
       break;
-    case isDirDocumentation:
+    case IndexSection::isDirDocumentation:
       m_t << "<chapter>\n";
       m_t << "    <title>";
       break;
-    case isNamespaceDocumentation:
+    case IndexSection::isNamespaceDocumentation:
       m_t << "<chapter>\n";
       m_t << "    <title>";
       break;
-    case isConceptDocumentation:
+    case IndexSection::isConceptDocumentation:
       m_t << "<chapter>\n";
       m_t << "    <title>";
       break;
-    case isClassDocumentation:
+    case IndexSection::isClassDocumentation:
       m_t << "<chapter>\n";
       m_t << "    <title>";
       break;
-    case isFileDocumentation:
+    case IndexSection::isFileDocumentation:
       m_t << "<chapter>\n";
       m_t << "    <title>";
       break;
-    case isExampleDocumentation:
+    case IndexSection::isExampleDocumentation:
       m_t << "<chapter>\n";
       m_t << "    <title>";
       break;
-    case isPageDocumentation:
+    case IndexSection::isPageDocumentation:
       break;
-    case isPageDocumentation2:
+    case IndexSection::isPageDocumentation2:
       break;
-    case isEndIndex:
+    case IndexSection::isEndIndex:
       break;
   }
 }
 
-void DocbookGenerator::endIndexSection(IndexSections is)
+void DocbookGenerator::endIndexSection(IndexSection is)
 {
-DB_GEN_C2("IndexSections " << is)
+DB_GEN_C2("IndexSection " << is)
   switch (is)
   {
-    case isTitlePageStart:
+    case IndexSection::isTitlePageStart:
       break;
-    case isTitlePageAuthor:
+    case IndexSection::isTitlePageAuthor:
       break;
-    case isMainPage:
-      m_t << "</title>\n";
-      m_t << "    <xi:include href=\"mainpage.xml\" xmlns:xi=\"http://www.w3.org/2001/XInclude\"/>\n";
-      m_t << "</chapter>\n";
+    case IndexSection::isMainPage:
+      {
+        if (Doxygen::mainPage)
+        {
+          writePageLink(QCString("mainpage"), TRUE);
+        }
+      }
       break;
-    case isModuleIndex:
+    case IndexSection::isModuleIndex:
       //m_t << "</chapter>\n";
       break;
-    case isDirIndex:
+    case IndexSection::isDirIndex:
       //m_t << "<xi:include href=\"dirs.xml\" xmlns:xi=\"http://www.w3.org/2001/XInclude\"/>";
       //m_t << "</chapter>\n";
       break;
-    case isNamespaceIndex:
+    case IndexSection::isNamespaceIndex:
       //m_t << "<xi:include href=\"namespaces.xml\" xmlns:xi=\"http://www.w3.org/2001/XInclude\"/>";
       //m_t << "</chapter>\n";
       break;
-    case isConceptIndex:
+    case IndexSection::isConceptIndex:
       //m_t << "<xi:include href=\"concepts.xml\" xmlns:xi=\"http://www.w3.org/2001/XInclude\"/>";
       //m_t << "</chapter>\n";
       break;
-    case isClassHierarchyIndex:
+    case IndexSection::isClassHierarchyIndex:
       //m_t << "<xi:include href=\"hierarchy.xml\" xmlns:xi=\"http://www.w3.org/2001/XInclude\"/>";
       //m_t << "</chapter>\n";
       break;
-    case isCompoundIndex:
+    case IndexSection::isCompoundIndex:
       //m_t << "</chapter>\n";
       break;
-    case isFileIndex:
+    case IndexSection::isFileIndex:
       //m_t << "<xi:include href=\"files.xml\" xmlns:xi=\"http://www.w3.org/2001/XInclude\"/>";
       //m_t << "</chapter>\n";
       break;
-    case isPageIndex:
+    case IndexSection::isPageIndex:
       //m_t << "<xi:include href=\"pages.xml\" xmlns:xi=\"http://www.w3.org/2001/XInclude\"/>";
       //m_t << "</chapter>\n";
       break;
-    case isModuleDocumentation:
+    case IndexSection::isModuleDocumentation:
       {
         m_t << "</title>\n";
         for (const auto &gd : *Doxygen::groupLinkedMap)
         {
-          if (!gd->isReference())
+          if (!gd->isReference() && !gd->isASubGroup())
           {
-            m_t << "    <xi:include href=\"" << gd->getOutputFileBase() << ".xml\" xmlns:xi=\"http://www.w3.org/2001/XInclude\"/>\n";
+            writePageLink(gd->getOutputFileBase(), TRUE);
           }
         }
       }
       m_t << "</chapter>\n";
       break;
-    case isDirDocumentation:
+    case IndexSection::isDirDocumentation:
       {
         m_t << "</title>\n";
         for (const auto &dd : *Doxygen::dirLinkedMap)
@@ -519,7 +575,7 @@ DB_GEN_C2("IndexSections " << is)
       }
       m_t << "</chapter>\n";
       break;
-    case isNamespaceDocumentation:
+    case IndexSection::isNamespaceDocumentation:
       {
         m_t << "</title>\n";
         for (const auto &nd : *Doxygen::namespaceLinkedMap)
@@ -532,7 +588,7 @@ DB_GEN_C2("IndexSections " << is)
       }
       m_t << "</chapter>\n";
       break;
-    case isConceptDocumentation:
+    case IndexSection::isConceptDocumentation:
       {
         m_t << "</title>\n";
         for (const auto &cd : *Doxygen::conceptLinkedMap)
@@ -545,7 +601,7 @@ DB_GEN_C2("IndexSections " << is)
       }
       m_t << "</chapter>\n";
       break;
-    case isClassDocumentation:
+    case IndexSection::isClassDocumentation:
       {
         m_t << "</title>\n";
         for (const auto &cd : *Doxygen::classLinkedMap)
@@ -562,7 +618,7 @@ DB_GEN_C2("IndexSections " << is)
       }
       m_t << "</chapter>\n";
       break;
-    case isFileDocumentation:
+    case IndexSection::isFileDocumentation:
       {
         m_t << "</title>\n";
         for (const auto &fn : *Doxygen::inputNameLinkedMap)
@@ -582,7 +638,7 @@ DB_GEN_C2("IndexSections " << is)
       }
       m_t << "</chapter>\n";
       break;
-    case isExampleDocumentation:
+    case IndexSection::isExampleDocumentation:
       {
         m_t << "</title>\n";
         for (const auto &pd : *Doxygen::exampleLinkedMap)
@@ -592,35 +648,34 @@ DB_GEN_C2("IndexSections " << is)
       }
       m_t << "</chapter>\n";
       break;
-    case isPageDocumentation:
+    case IndexSection::isPageDocumentation:
+        for (const auto &pd : *Doxygen::pageLinkedMap)
+        {
+          if (!pd->getGroupDef() && !pd->isReference() && !pd->hasParentPage()
+            && Doxygen::mainPage.get() != pd.get())
+          {
+            writePageLink(pd->getOutputFileBase(), TRUE);
+          }
+        }
       break;
-    case isPageDocumentation2:
+    case IndexSection::isPageDocumentation2:
       break;
-    case isEndIndex:
+    case IndexSection::isEndIndex:
       m_t << "<index/>\n";
       break;
   }
 }
-void DocbookGenerator::writePageLink(const QCString &name, bool /*first*/)
+void DocbookGenerator::writePageLink(const QCString &name, bool first)
 {
 DB_GEN_C
-  for (const auto &pd : *Doxygen::pageLinkedMap)
-  {
-    if (!pd->getGroupDef() && !pd->isReference() && pd->name() == stripPath(name))
-    {
-      m_t << "<chapter>\n";
-      if (pd->hasTitle())
-      {
-        m_t << "    <title>" << convertToDocBook(pd->title()) << "</title>\n";
-      }
-      else
-      {
-        m_t << "    <title>" << convertToDocBook(pd->name()) << "</title>\n";
-      }
-      m_t << "    <xi:include href=\"" << pd->getOutputFileBase() << ".xml\" xmlns:xi=\"http://www.w3.org/2001/XInclude\"/>\n";
-      m_t << "</chapter>\n";
-    }
-  }
+  QCString link;
+  link.sprintf("    <xi:include href=\"%s.xml\" xmlns:xi=\"http://www.w3.org/2001/XInclude\"/>\n",
+               name.data());
+  if (first)
+    m_t << link;
+  else
+    // Buffer page links and write them after all sections are closed
+    m_pageLinks += link;
 }
 
 void DocbookGenerator::writeDoc(const IDocNodeAST *ast,const Definition *ctx,const MemberDef *,int)
@@ -629,7 +684,7 @@ DB_GEN_C
   auto astImpl = dynamic_cast<const DocNodeAST*>(ast);
   if (astImpl)
   {
-    auto visitor { DocbookDocVisitor(m_t,*this,ctx?ctx->getDefFileExtension():QCString()) };
+    DocbookDocVisitor visitor(m_t,*m_codeList,ctx?ctx->getDefFileExtension():QCString());
     std::visit(visitor,astImpl->root);
   }
 }
@@ -668,21 +723,29 @@ void DocbookGenerator::docify(const QCString &str)
 DB_GEN_C
   m_t << convertToDocBook(str);
 }
-void DocbookGenerator::writeObjectLink(const QCString &, const QCString &f,
-                                     const QCString &anchor, const QCString &text)
+static QCString objectLinkToString(const QCString &, const QCString &f,
+                                   const QCString &anchor, const QCString &text)
 {
 DB_GEN_C
+  QCString result;
   if (!anchor.isEmpty())
   {
-    if (!f.isEmpty()) m_t << "<link linkend=\"_" << stripPath(f) << "_1" << anchor << "\">";
-    else   m_t << "<link linkend=\"_" << anchor << "\">";
+    if (!f.isEmpty()) result += "<link linkend=\"_" + stripPath(f) + "_1" + anchor + "\">";
+    else   result += "<link linkend=\"_" + anchor + "\">";
   }
   else
   {
-    m_t << "<link linkend=\"_" << stripPath(f) << "\">";
+    result += "<link linkend=\"_" + stripPath(f) + "\">";
   }
-  docify(text);
-  m_t << "</link>";
+  result += convertToDocBook(text);
+  result += "</link>";
+  return result;
+}
+void DocbookGenerator::writeObjectLink(const QCString &ref, const QCString &f,
+                                     const QCString &anchor, const QCString &text)
+{
+DB_GEN_C
+  m_t << objectLinkToString(ref,f,anchor,text);
 }
 void DocbookGenerator::startMemberList()
 {
@@ -700,14 +763,14 @@ DB_GEN_C
   if (m_inSimpleSect[m_levelListItem]) m_t << "</simplesect>\n";
   m_inSimpleSect[m_levelListItem] = FALSE;
 }
-void DocbookGenerator::startMemberItem(const QCString &,int,const QCString &)
+void DocbookGenerator::startMemberItem(const QCString &,MemberItemType,const QCString &)
 {
 DB_GEN_C
   if (m_inListItem[m_levelListItem]) m_t << "</listitem>\n";
   m_t << "            <listitem><para>";
   m_inListItem[m_levelListItem] = TRUE;
 }
-void DocbookGenerator::endMemberItem()
+void DocbookGenerator::endMemberItem(MemberItemType)
 {
 DB_GEN_C
   m_t << "</para>\n";
@@ -925,12 +988,12 @@ void DocbookGenerator::endExamples()
 DB_GEN_C
   m_t << "</simplesect>\n";
 }
-void DocbookGenerator::startSubsubsection()
+void DocbookGenerator::startCompoundTemplateParams()
 {
 DB_GEN_C
   m_t << "<simplesect><title>";
 }
-void DocbookGenerator::endSubsubsection()
+void DocbookGenerator::endCompoundTemplateParams()
 {
 DB_GEN_C
   m_t << "</title></simplesect>\n";
@@ -1034,7 +1097,7 @@ DB_GEN_C
   m_simpleTable = true;
 }
 
-void DocbookGenerator::endMemberDocSimple(bool isEnum)
+void DocbookGenerator::endMemberDocSimple(bool /* isEnum */)
 {
 DB_GEN_C
   m_t << "    </tbody>\n";
@@ -1253,3 +1316,134 @@ void DocbookGenerator::closeAllSections()
   }
 }
 
+void DocbookGenerator::writeInheritedSectionTitle(
+                  const QCString &/*id*/,const QCString &ref,
+                  const QCString &file,  const QCString &anchor,
+                  const QCString &title, const QCString &name)
+{
+DB_GEN_C
+  m_t << theTranslator->trInheritedFrom(convertToDocBook(title), objectLinkToString(ref, file, anchor, name));
+}
+
+void DocbookGenerator::writeLocalToc(const SectionRefs &sectionRefs,const LocalToc &localToc)
+{
+  if (localToc.isDocbookEnabled())
+  {
+    m_t << "    <toc>\n";
+    m_t << "    <title>" << theTranslator->trRTFTableOfContents() << "</title>\n";
+    int level=1,l;
+    int maxLevel = localToc.docbookLevel();
+    BoolVector inLi(maxLevel+1,false);
+    for (const SectionInfo *si : sectionRefs)
+    {
+      SectionType type = si->type();
+      if (isSection(type))
+      {
+        //printf("  level=%d title=%s\n",level,qPrint(si->title));
+        int nextLevel = static_cast<int>(type);
+        if (nextLevel>level)
+        {
+          for (l=level;l<nextLevel;l++)
+          {
+            if (l < maxLevel) m_t << "    <tocdiv>\n";
+          }
+        }
+        else if (nextLevel<level)
+        {
+          for (l=level;l>nextLevel;l--)
+          {
+            inLi[l]=FALSE;
+            if (l <= maxLevel) m_t << "    </tocdiv>\n";
+          }
+        }
+        if (nextLevel <= maxLevel)
+        {
+          QCString titleDoc = convertToDocBook(si->title());
+          QCString label    = convertToDocBook(si->label());
+          if (titleDoc.isEmpty()) titleDoc = label;
+          m_t << "      <tocentry>" << titleDoc << "</tocentry>\n";
+        }
+        inLi[nextLevel]=TRUE;
+        level = nextLevel;
+      }
+    }
+    if (level > maxLevel) level = maxLevel;
+    while (level>1 && level <= maxLevel)
+    {
+      inLi[level]=FALSE;
+      m_t << "</tocdiv>\n";
+      level--;
+    }
+    m_t << "    </toc>\n";
+  }
+}
+
+//-------------------------------------------------------------------------------------------------
+
+static constexpr auto hex="0123456789ABCDEF";
+
+/*! Converts a string to an DocBook-encoded string */
+QCString convertToDocBook(const QCString &s, const bool retainNewline)
+{
+  if (s.isEmpty()) return s;
+  GrowBuf growBuf;
+  const char *q;
+  int cnt;
+  const char *p=s.data();
+  char c;
+  while ((c=*p++))
+  {
+    switch (c)
+    {
+      case '\n': if (retainNewline) growBuf.addStr("<literallayout>&#160;&#xa;</literallayout>"); growBuf.addChar(c);   break;
+      case '<':  growBuf.addStr("&lt;");   break;
+      case '>':  growBuf.addStr("&gt;");   break;
+      case '&':  // possibility to have a special symbol
+        q = p;
+        cnt = 2; // we have to count & and ; as well
+        while ((*q >= 'a' && *q <= 'z') || (*q >= 'A' && *q <= 'Z') || (*q >= '0' && *q <= '9'))
+        {
+          cnt++;
+          q++;
+        }
+        if (*q == ';')
+        {
+           --p; // we need & as well
+           HtmlEntityMapper::SymType res = HtmlEntityMapper::instance().name2sym(QCString(p).left(cnt));
+           if (res == HtmlEntityMapper::Sym_Unknown)
+           {
+             p++;
+             growBuf.addStr("&amp;");
+           }
+           else
+           {
+             growBuf.addStr(HtmlEntityMapper::instance().docbook(res));
+             q++;
+             p = q;
+           }
+        }
+        else
+        {
+          growBuf.addStr("&amp;");
+        }
+        break;
+      case '\'': growBuf.addStr("&apos;"); break;
+      case '"':  growBuf.addStr("&quot;"); break;
+      case  1: case  2: case  3: case  4: case  5: case  6: case 7:  case  8:
+      case 11: case 12: case 14: case 15: case 16: case 17: case 18:
+      case 19: case 20: case 21: case 22: case 23: case 24: case 25: case 26:
+      case 27: case 28: case 29: case 30: case 31:
+        growBuf.addStr("&#x24");
+        growBuf.addChar(hex[static_cast<uint8_t>(c)>>4]);
+        growBuf.addChar(hex[static_cast<uint8_t>(c)&0xF]);
+        growBuf.addChar(';');
+        break;
+      default:
+        growBuf.addChar(c);
+        break;
+    }
+  }
+  growBuf.addChar(0);
+  return growBuf.get();
+}
+
