@@ -1,13 +1,10 @@
 /******************************************************************************
  *
- * $Id: image.cpp,v 1.14 2001/03/19 19:27:40 root Exp $
- *
- *
- * Copyright (C) 1997-2010 by Dimitri van Heesch.
+ * Copyright (C) 1997-2022 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
- * documentation under the terms of the GNU General Public License is hereby 
- * granted. No representations are made about the suitability of this software 
+ * documentation under the terms of the GNU General Public License is hereby
+ * granted. No representations are made about the suitability of this software
  * for any purpose. It is provided "as is" without express or implied warranty.
  * See the GNU General Public License for more details.
  *
@@ -16,16 +13,16 @@
  *
  */
 
-#include "qtbc.h"
+#include <vector>
+#include <cmath>
+
 #include "image.h"
-//#include "gifenc.h"
-#include <qfile.h>
-#include <math.h>
 #include "lodepng.h"
 #include "config.h"
 
 typedef unsigned char  Byte;
 
+/** Helper struct representing a RGBA color */
 struct Color
 {
   Byte red;
@@ -38,10 +35,10 @@ const int charSetWidth=80;
 const int charHeight=12;
 const int numChars=96;
 
-unsigned short charPos[numChars]    = 
-  {   
-      0,  5,  8, 13, 20, 27, 38, 47, 
-     50, 54, 58, 65, 72, 76, 83, 87, 
+const unsigned short charPos[numChars]    =
+  {
+      0,  5,  8, 13, 20, 27, 38, 47,
+     50, 54, 58, 65, 72, 76, 83, 87,
      91, 98,105,112,119,126,133,140,
     147,154,161,164,167,174,181,188,
     195,207,216,224,233,242,250,258,
@@ -54,7 +51,7 @@ unsigned short charPos[numChars]    =
     594,600,607,613,617,620,624,631
   };
 
-unsigned char charWidth[numChars] = 
+const unsigned char charWidth[numChars] =
   {
      5, 3, 5, 7, 7,11, 9, 3,
      4, 4, 7, 7, 4, 7, 4, 4,
@@ -68,10 +65,10 @@ unsigned char charWidth[numChars] =
      3, 7, 7, 7, 7, 7, 4, 7,
      7, 3, 3, 6, 3, 9, 7, 7,
      7, 6, 5, 6, 3, 7, 7, 9,
-     6, 7, 6, 4, 3, 4, 7, 5   
+     6, 7, 6, 4, 3, 4, 7, 5
   };
 
-unsigned char fontRaw[charSetWidth*charHeight] = {
+const unsigned char fontRaw[charSetWidth*charHeight] = {
   0x02, 0x50, 0x01, 0x06, 0x20, 0x60, 0xc6, 0x04, 0x00, 0x00, 0x00, 0x27,
   0x04, 0x1c, 0x38, 0x11, 0xf1, 0xc7, 0xc7, 0x0e, 0x00, 0x00, 0x00, 0x03,
   0x81, 0xf0, 0x10, 0x7c, 0x1e, 0x3e, 0x1f, 0x9f, 0x87, 0x88, 0x24, 0x09,
@@ -154,65 +151,31 @@ unsigned char fontRaw[charSetWidth*charHeight] = {
   0x40, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x01, 0xac, 0x00, 0x00
 };
 
-static Color palette[] =
+
+struct Image::Private
 {
-  { 0xff, 0xff, 0xff, 0x00 },
-  { 0x00, 0x00, 0x00, 0xff },
-  { 0xff, 0xff, 0xc0, 0xff },
-  { 0x9f, 0x9f, 0x60, 0xff },
-  { 0x90, 0x00, 0x00, 0xff },
-  { 0x00, 0x90, 0x00, 0xff },
-  { 0x00, 0x00, 0x90, 0xff },
-  { 0xc0, 0xc0, 0xc0, 0xff }
+  uint32_t width;
+  uint32_t height;
+  std::vector<uint8_t> data;
+  std::vector<Color> palette =
+  {
+    { 0xff, 0xff, 0xff, 0x00 },
+    { 0x00, 0x00, 0x00, 0xff },
+    { 0xff, 0xff, 0xc0, 0xff },
+    { 0x9f, 0x9f, 0x60, 0xff },
+    { 0xa7, 0x38, 0x30, 0xff },
+    { 0x29, 0x70, 0x18, 0xff },
+    { 0x97, 0xCC, 0xE8, 0xff },
+    { 0xc0, 0xc0, 0xc0, 0xff },
+    { 0xff, 0xff, 0xff, 0xff }
+  };
 };
 
-// for alpha we use x^(1/1.3)
-static Color palette2[] =
+Image::Image(uint32_t w,uint32_t h) : p(std::make_unique<Private>())
 {
-  { 0x00, 0x00, 0x00, 0x00 },
-  { 0x00, 0x00, 0x00, 0x2e },
-  { 0x00, 0x00, 0x00, 0x48 },
-  { 0x00, 0x00, 0x00, 0x5d },
-  { 0x00, 0x00, 0x00, 0x6f },
-  { 0x00, 0x00, 0x00, 0x80 },
-  { 0x00, 0x00, 0x00, 0x8f },
-  { 0x00, 0x00, 0x00, 0x9e },
-  { 0x00, 0x00, 0x00, 0xac },
-  { 0x00, 0x00, 0x00, 0xb9 },
-  { 0x00, 0x00, 0x00, 0xc5 },
-  { 0x00, 0x00, 0x00, 0xd2 },
-  { 0x00, 0x00, 0x00, 0xdd },
-  { 0x00, 0x00, 0x00, 0xe9 },
-  { 0x00, 0x00, 0x00, 0xf4 },
-  { 0x00, 0x00, 0x00, 0xff }
-};
-
-static Color palette3[] =
-{
-  { 0xff, 0xff, 0xff, 0xff },
-  { 0xe0, 0xe0, 0xe0, 0xff },
-  { 0xd0, 0xd0, 0xd0, 0xff },
-  { 0xc0, 0xc0, 0xc0, 0xff },
-  { 0xb0, 0xb0, 0xb0, 0xff },
-  { 0xa0, 0xa0, 0xa0, 0xff },
-  { 0x90, 0x90, 0x90, 0xff },
-  { 0x80, 0x80, 0x80, 0xff },
-  { 0x70, 0x70, 0x70, 0xff },
-  { 0x60, 0x60, 0x60, 0xff },
-  { 0x50, 0x50, 0x50, 0xff },
-  { 0x40, 0x40, 0x40, 0xff },
-  { 0x30, 0x30, 0x30, 0xff },
-  { 0x20, 0x20, 0x20, 0xff },
-  { 0x10, 0x10, 0x10, 0xff },
-  { 0x00, 0x00, 0x00, 0xff }
-};
-
-
-Image::Image(int w,int h)
-{
-  static int hue   = Config_getInt("HTML_COLORSTYLE_HUE");
-  static int sat   = Config_getInt("HTML_COLORSTYLE_SAT");
-  static int gamma = Config_getInt("HTML_COLORSTYLE_GAMMA");
+  int hue   = Config_getInt(HTML_COLORSTYLE_HUE);
+  int sat   = Config_getInt(HTML_COLORSTYLE_SAT);
+  int gamma = Config_getInt(HTML_COLORSTYLE_GAMMA);
 
   double red1,green1,blue1;
   double red2,green2,blue2;
@@ -229,65 +192,60 @@ Image::Image(int w,int h)
                         &red2,&green2,&blue2
                        );
 
-  palette[2].red   = (int)(red1   * 255.0);
-  palette[2].green = (int)(green1 * 255.0);
-  palette[2].blue  = (int)(blue1  * 255.0);
+  p->palette[2].red   = static_cast<Byte>(red1   * 255.0);
+  p->palette[2].green = static_cast<Byte>(green1 * 255.0);
+  p->palette[2].blue  = static_cast<Byte>(blue1  * 255.0);
 
-  palette[3].red   = (int)(red2   * 255.0);
-  palette[3].green = (int)(green2 * 255.0);
-  palette[3].blue  = (int)(blue2  * 255.0);
+  p->palette[3].red   = static_cast<Byte>(red2   * 255.0);
+  p->palette[3].green = static_cast<Byte>(green2 * 255.0);
+  p->palette[3].blue  = static_cast<Byte>(blue2  * 255.0);
 
-  data = new uchar[w*h];
-  memset(data,0,w*h);
-  width = w;
-  height = h;
+  p->data.resize(w*h);
+  p->width = w;
+  p->height = h;
 }
 
-Image::~Image()
+uint32_t Image::width() const  { return p->width; }
+uint32_t Image::height() const { return p->height; }
+
+Image::~Image() = default;
+
+void Image::setPixel(uint32_t x,uint32_t y,uint8_t val)
 {
-  delete[] data;
+  if (x<p->width && y<p->height) p->data[y*p->width+x] = val;
 }
 
-void Image::setPixel(int x,int y,uchar val)
+uint8_t Image::getPixel(uint32_t x,uint32_t y) const
 {
-  if (x>=0 && x<width && y>=0 && y<height)
-    data[y*width+x] = val;
+  return (x<p->width && y<p->height) ? p->data[y*p->width+x] : 0;
 }
 
-uchar Image::getPixel(int x,int y) const
-{
-  if (x>=0 && x<width && y>=0 && y<height)
-    return data[y*width+x];
-  else
-    return 0;
-}
-
-void Image::writeChar(int x,int y,char c,uchar fg) 
+void Image::writeChar(uint32_t x,uint32_t y,char c,uint8_t fg)
 {
   if (c>=' ')
   {
-    int xf,yf,ci=c-' ';
-    int rowOffset=0;
-    int cw = charWidth[ci];
-    int cp = charPos[ci];
+    uint32_t xf,yf,ci=c-' ';
+    uint32_t rowOffset=0;
+    uint32_t cw = charWidth[ci];
+    uint32_t cp = charPos[ci];
     for (yf=0;yf<charHeight;yf++)
     {
       unsigned short bitPattern=0;
-      int bitsLeft=cw;
-      int byteOffset = rowOffset+(cp>>3);
-      int bitOffset  = cp&7;
+      uint32_t bitsLeft=cw;
+      uint32_t byteOffset = rowOffset+(cp>>3);
+      uint32_t bitOffset  = cp&7;
       // get the bit pattern for row yf of the character from the font data
       while (bitsLeft>0)
       {
-        int bits=8-bitOffset;
-        if (bits>bitsLeft) bits=bitsLeft; 
-        bitPattern<<=bits; 
+        uint32_t bits=8-bitOffset;
+        if (bits>bitsLeft) bits=bitsLeft;
+        bitPattern<<=bits;
         bitPattern|=((fontRaw[byteOffset]<<bitOffset)&0xff)>>(8-bits);
         bitsLeft-=bits;
         bitOffset=0;
         byteOffset++;
       }
-      int mask=1<<(cw-1);
+      uint32_t mask=1<<(cw-1);
       // draw character row yf
       for (xf=0;xf<cw;xf++)
       {
@@ -296,15 +254,16 @@ void Image::writeChar(int x,int y,char c,uchar fg)
       }
       rowOffset+=charSetWidth;
     }
-  } 
+  }
 }
 
-void Image::writeString(int x,int y,const char *s,uchar fg) 
+void Image::writeString(uint32_t x,uint32_t y,const QCString &s,uint8_t fg)
 {
-  if (s)
+  if (!s.isEmpty())
   {
+    const char *ps = s.data();
     char c;
-    while ((c=*s++))
+    while ((c=*ps++))
     {
       writeChar(x,y,c,fg);
       x+=charWidth[c-' '];
@@ -312,59 +271,60 @@ void Image::writeString(int x,int y,const char *s,uchar fg)
   }
 }
 
-uint Image::stringLength(const char *s) 
+uint32_t Image::stringLength(const QCString &s)
 {
-  int w=0;
-  if (s)
+  uint32_t w=0;
+  if (!s.isEmpty())
   {
+    const char *ps = s.data();
     char c;
-    while ((c=*s++)) w+=charWidth[c-' '];
+    while ((c=*ps++)) w+=charWidth[c-' '];
   }
   return w;
 }
 
-void Image::drawHorzLine(int y,int xs,int xe,uchar colIndex,uint mask)
+void Image::drawHorzLine(uint32_t y,uint32_t xs,uint32_t xe,uint8_t colIndex,uint32_t mask)
 {
-  int x,i=0,j=0;
-  for (x=xs;x<=xe;x++,j++) 
+  uint32_t x,i=0,j=0;
+  for (x=xs;x<=xe;x++,j++)
   {
     if (j&1) i++;
     if (mask&(1<<(i&0x1f))) setPixel(x,y,colIndex);
   }
-} 
+}
 
-void Image::drawHorzArrow(int y,int xs,int xe,uchar colIndex,uint mask)
+void Image::drawHorzArrow(uint32_t y,uint32_t xs,uint32_t xe,uint8_t colIndex,uint32_t mask)
 {
   drawHorzLine(y,xs,xe,colIndex,mask);
-  int i;
+  uint32_t i;
   for (i=0;i<6;i++)
   {
-    int h=i>>1;
+    uint32_t h=i>>1;
     drawVertLine(xe-i,y-h,y+h,colIndex,0xffffffff);
   }
-} 
+}
 
-void Image::drawVertLine(int x,int ys,int ye,uchar colIndex,uint mask)
+void Image::drawVertLine(uint32_t x,uint32_t ys,uint32_t ye,uint8_t colIndex,uint32_t mask)
 {
-  int y,i=0;
-  for (y=ys;y<=ye;y++,i++) 
+  uint32_t y,i=0;
+  for (y=ys;y<=ye;y++,i++)
   {
     if (mask&(1<<(i&0x1f))) setPixel(x,y,colIndex);
   }
 }
 
-void Image::drawVertArrow(int x,int ys,int ye,uchar colIndex,uint mask)
+void Image::drawVertArrow(uint32_t x,uint32_t ys,uint32_t ye,uint8_t colIndex,uint32_t mask)
 {
   drawVertLine(x,ys,ye,colIndex,mask);
-  int i;
+  uint32_t i;
   for (i=0;i<6;i++)
   {
-    int h=i>>1;
+    uint32_t h=i>>1;
     drawHorzLine(ys+i,x-h,x+h,colIndex,0xffffffff);
   }
 }
 
-void Image::drawRect(int x,int y,int w,int h,uchar colIndex,uint mask)
+void Image::drawRect(uint32_t x,uint32_t y,uint32_t w,uint32_t h,uint8_t colIndex,uint32_t mask)
 {
   drawHorzLine(y,x,x+w-1,colIndex,mask);
   drawHorzLine(y+h-1,x,x+w-1,colIndex,mask);
@@ -372,53 +332,30 @@ void Image::drawRect(int x,int y,int w,int h,uchar colIndex,uint mask)
   drawVertLine(x+w-1,y,y+h-1,colIndex,mask);
 }
 
-void Image::fillRect(int x,int y,int lwidth,int lheight,uchar colIndex,uint mask)
+void Image::fillRect(uint32_t x,uint32_t y,uint32_t width,uint32_t height,uint8_t colIndex,uint32_t mask)
 {
-  int xp,yp,xi,yi;
-  for (yp=y,yi=0;yp<y+lheight;yp++,yi++)
-    for (xp=x,xi=0;xp<x+lwidth;xp++,xi++)
-      if (mask&(1<<((xi+yi)&0x1f))) 
+  uint32_t xp,yp,xi,yi;
+  for (yp=y,yi=0;yp<y+height;yp++,yi++)
+    for (xp=x,xi=0;xp<x+width;xp++,xi++)
+      if (mask&(1<<((xi+yi)&0x1f)))
         setPixel(xp,yp,colIndex);
 }
 
-bool Image::save(const char *fileName,int mode)
+bool Image::save(const QCString &fileName)
 {
-#if 0
-  GifEncoder gifenc(data,
-                    mode==0 ? palette : palette2,
-                    width,height,
-                    mode==0 ? 3 : 4,
-                    0);
-  QFile file(fileName);
-  if (file.open(IO_WriteOnly))
-  {
-    gifenc.writeGIF(file);
-    return TRUE;
-  }
-  else
-  {
-    return FALSE;
-  }
-#endif
-  static bool useTransparency = Config_getBool("FORMULA_TRANSPARENT");
-  uchar* buffer;
+  uint8_t* buffer;
   size_t bufferSize;
   LodePNG_Encoder encoder;
   LodePNG_Encoder_init(&encoder);
-  int numCols = mode==0 ? 8 : 16;
-  Color *pPal = mode==0         ? palette  : 
-                useTransparency ? palette2 : 
-                                  palette3 ;
-  int i;
-  for (i=0;i<numCols;i++,pPal++)
+  for (const auto &col : p->palette)
   {
     LodePNG_InfoColor_addPalette(&encoder.infoPng.color,
-                                 pPal->red,pPal->green,pPal->blue,pPal->alpha);
+                                 col.red,col.green,col.blue,col.alpha);
   }
-  encoder.infoPng.color.colorType = 3; 
+  encoder.infoPng.color.colorType = 3;
   encoder.infoRaw.color.colorType = 3;
-  LodePNG_encode(&encoder, &buffer, &bufferSize, data, width, height);
-  LodePNG_saveFile(buffer, bufferSize, fileName);
+  LodePNG_encode(&encoder, &buffer, &bufferSize, &p->data[0], p->width, p->height);
+  LodePNG_saveFile(buffer, bufferSize, fileName.data());
   free(buffer);
   LodePNG_Encoder_cleanup(&encoder);
   return TRUE;
@@ -446,7 +383,7 @@ void ColoredImage::hsl2rgb(double h,double s,double l,
     m       = l + l - v;
     sv      = (v - m ) / v;
     h      *= 6.0;
-    sextant = (int)h;
+    sextant = static_cast<int>(h);
     fract   = h - sextant;
     vsf     = v * sv * fract;
     mid1    = m + vsf;
@@ -490,49 +427,54 @@ void ColoredImage::hsl2rgb(double h,double s,double l,
   *pBlue  = b;
 }
 
-ColoredImage::ColoredImage(int width,int height,
-           const uchar *greyLevels,const uchar *alphaLevels,
-           int saturation,int hue,int gamma)
+struct ColoredImage::Private
 {
-  m_hasAlpha = alphaLevels!=0;
-  m_width    = width;
-  m_height   = height;
-  m_data     = (uchar*)malloc(width*height*4);
-  int i;
+  uint32_t width;
+  uint32_t height;
+  std::vector<uint8_t> data;
+  bool hasAlpha;
+};
+
+ColoredImage::ColoredImage(uint32_t width,uint32_t height,
+           const uint8_t *greyLevels,const uint8_t *alphaLevels,
+           int saturation,int hue,int gamma) : p(std::make_unique<Private>())
+{
+  p->hasAlpha = alphaLevels!=0;
+  p->width    = width;
+  p->height   = height;
+  p->data.resize(width*height*4);
+  uint32_t i;
   for (i=0;i<width*height;i++)
   {
-    uchar r,g,b,a;
+    Byte r,g,b,a;
     double red,green,blue;
     hsl2rgb(hue/360.0,                            // hue
             saturation/255.0,                     // saturation
             pow(greyLevels[i]/255.0,gamma/100.0), // luma (gamma corrected)
             &red,&green,&blue);
-    r = (int)(red  *255.0);
-    g = (int)(green*255.0);
-    b = (int)(blue *255.0);
+    r = static_cast<Byte>(red  *255.0);
+    g = static_cast<Byte>(green*255.0);
+    b = static_cast<Byte>(blue *255.0);
     a = alphaLevels ? alphaLevels[i] : 255;
-    m_data[i*4+0]=r;
-    m_data[i*4+1]=g;
-    m_data[i*4+2]=b;
-    m_data[i*4+3]=a;
+    p->data[i*4+0]=r;
+    p->data[i*4+1]=g;
+    p->data[i*4+2]=b;
+    p->data[i*4+3]=a;
   }
 }
 
-ColoredImage::~ColoredImage()
-{
-  free(m_data);
-}
+ColoredImage::~ColoredImage() = default;
 
-bool ColoredImage::save(const char *fileName)
+bool ColoredImage::save(const QCString &fileName)
 {
-  uchar *buffer;
+  uint8_t *buffer;
   size_t bufferSize;
   LodePNG_Encoder encoder;
   LodePNG_Encoder_init(&encoder);
-  encoder.infoPng.color.colorType = m_hasAlpha ? 6 : 2; // 2=RGB 24 bit, 6=RGBA 32 bit
+  encoder.infoPng.color.colorType = p->hasAlpha ? 6 : 2; // 2=RGB 24 bit, 6=RGBA 32 bit
   encoder.infoRaw.color.colorType = 6; // 6=RGBA 32 bit
-  LodePNG_encode(&encoder, &buffer, &bufferSize, m_data, m_width, m_height);
-  LodePNG_saveFile(buffer, bufferSize, fileName);
+  LodePNG_encode(&encoder, &buffer, &bufferSize, &p->data[0], p->width, p->height);
+  LodePNG_saveFile(buffer, bufferSize, fileName.data());
   LodePNG_Encoder_cleanup(&encoder);
   free(buffer);
   return TRUE;
