@@ -859,14 +859,14 @@ class TextGeneratorSqlite3Impl : public TextGeneratorIntf
 };
 
 
-static bool bindTextParameter(SqlStmt &s,const char *name,const QCString &value, bool _static=FALSE)
+static bool bindTextParameter(SqlStmt &s,const char *name,const QCString &value)
 {
   int idx = sqlite3_bind_parameter_index(s.stmt, name);
   if (idx==0) {
     err("sqlite3_bind_parameter_index(%s)[%s] failed: %s\n", name, s.query, sqlite3_errmsg(s.db));
     return false;
   }
-  int rv = sqlite3_bind_text(s.stmt, idx, value.data(), -1, _static==TRUE?SQLITE_STATIC:SQLITE_TRANSIENT);
+  int rv = sqlite3_bind_text(s.stmt, idx, value.data(), -1, SQLITE_TRANSIENT);
   if (rv!=SQLITE_OK) {
     err("sqlite3_bind_text(%s)[%s] failed: %s\n", name, s.query, sqlite3_errmsg(s.db));
     return false;
@@ -930,7 +930,7 @@ static int insertPath(QCString name, bool local=TRUE, bool found=TRUE, int type=
 static void recordMetadata()
 {
   bindTextParameter(meta_insert,":doxygen_version",getFullVersion());
-  bindTextParameter(meta_insert,":schema_version","0.2.1",TRUE); //TODO: this should be a constant somewhere; not sure where
+  bindTextParameter(meta_insert,":schema_version","0.2.1"); //TODO: this should be a constant somewhere; not sure where
   bindTextParameter(meta_insert,":generated_at",dateToString(DateTimeType::DateTime));
   bindTextParameter(meta_insert,":generated_on",dateToString(DateTimeType::Date));
   bindTextParameter(meta_insert,":project_name",Config_getString(PROJECT_NAME));
@@ -1128,8 +1128,8 @@ static void associateMember(const MemberDef *md, struct Refid member_refid, stru
     bindIntParameter(member_insert, ":scope_rowid", scope_refid.rowid);
     bindIntParameter(member_insert, ":memberdef_rowid", member_refid.rowid);
 
-    bindIntParameter(member_insert, ":prot", md->protection());
-    bindIntParameter(member_insert, ":virt", md->virtualness());
+    bindIntParameter(member_insert, ":prot", static_cast<int>(md->protection()));
+    bindIntParameter(member_insert, ":virt", static_cast<int>(md->virtualness()));
     step(member_insert);
   }
 }
@@ -1426,9 +1426,10 @@ QCString getSQLDocBlock(const Definition *scope,
   auto astImpl = dynamic_cast<const DocNodeAST*>(ast.get());
   if (astImpl)
   {
-    XMLCodeGenerator codeGen(t);
+    OutputCodeList xmlCodeList;
+    xmlCodeList.add<XMLCodeGenerator>(&t);
     // create a parse tree visitor for XML
-    XmlDocVisitor visitor(t,codeGen,
+    XmlDocVisitor visitor(t,xmlCodeList,
         scope ? scope->getDefFileExtension() : QCString(""));
     std::visit(visitor,astImpl->root);
   }
@@ -1630,7 +1631,7 @@ static void generateSqlite3ForMember(const MemberDef *md, struct Refid scope_ref
 
   bindIntParameter(memberdef_insert,":rowid", refid.rowid);
   bindTextParameter(memberdef_insert,":kind",md->memberTypeName());
-  bindIntParameter(memberdef_insert,":prot",md->protection());
+  bindIntParameter(memberdef_insert,":prot",static_cast<int>(md->protection()));
 
   bindIntParameter(memberdef_insert,":static",md->isStatic());
   bindIntParameter(memberdef_insert,":extern",md->isExternal());
@@ -1662,7 +1663,7 @@ static void generateSqlite3ForMember(const MemberDef *md, struct Refid scope_ref
     bindIntParameter(memberdef_insert,":optional",md->isOptional());
     bindIntParameter(memberdef_insert,":required",md->isRequired());
 
-    bindIntParameter(memberdef_insert,":virt",md->virtualness());
+    bindIntParameter(memberdef_insert,":virt",static_cast<int>(md->virtualness()));
   }
 
   if (md->memberType() == MemberType_Variable)
@@ -1922,7 +1923,7 @@ static void generateSqlite3ForClass(const ClassDef *cd)
   bindTextParameter(compounddef_insert,":name",cd->name());
   bindTextParameter(compounddef_insert,":title",cd->title());
   bindTextParameter(compounddef_insert,":kind",cd->compoundTypeString());
-  bindIntParameter(compounddef_insert,":prot",cd->protection());
+  bindIntParameter(compounddef_insert,":prot",static_cast<int>(cd->protection()));
 
   int file_id = insertPath(cd->getDefFileName());
   bindIntParameter(compounddef_insert,":file_id",file_id);
@@ -1991,8 +1992,8 @@ static void generateSqlite3ForClass(const ClassDef *cd)
     struct Refid derived_refid = insertRefid(cd->getOutputFileBase());
     bindIntParameter(compoundref_insert,":base_rowid", base_refid.rowid);
     bindIntParameter(compoundref_insert,":derived_rowid", derived_refid.rowid);
-    bindIntParameter(compoundref_insert,":prot",bcd.prot);
-    bindIntParameter(compoundref_insert,":virt",bcd.virt);
+    bindIntParameter(compoundref_insert,":prot",static_cast<int>(bcd.prot));
+    bindIntParameter(compoundref_insert,":virt",static_cast<int>(bcd.virt));
     step(compoundref_insert);
   }
 
@@ -2003,8 +2004,8 @@ static void generateSqlite3ForClass(const ClassDef *cd)
     struct Refid base_refid = insertRefid(cd->getOutputFileBase());
     bindIntParameter(compoundref_insert,":base_rowid", base_refid.rowid);
     bindIntParameter(compoundref_insert,":derived_rowid", derived_refid.rowid);
-    bindIntParameter(compoundref_insert,":prot",bcd.prot);
-    bindIntParameter(compoundref_insert,":virt",bcd.virt);
+    bindIntParameter(compoundref_insert,":prot",static_cast<int>(bcd.prot));
+    bindIntParameter(compoundref_insert,":virt",static_cast<int>(bcd.virt));
     step(compoundref_insert);
   }
 
@@ -2305,7 +2306,7 @@ static void generateSqlite3ForGroup(const GroupDef *gd)
   // + classes
   writeInnerClasses(gd->getClasses(),refid);
 
-  // + concepts 
+  // + concepts
   writeInnerConcepts(gd->getConcepts(),refid);
 
   // + namespaces
@@ -2436,7 +2437,7 @@ static void generateSqlite3ForPage(const PageDef *pd,bool isExample)
   // + title
   bindTextParameter(compounddef_insert,":title",title);
 
-  bindTextParameter(compounddef_insert,":kind", isExample ? "example" : "page",TRUE);
+  bindTextParameter(compounddef_insert,":kind", isExample ? "example" : "page");
 
   int file_id = insertPath(pd->getDefFileName());
 
