@@ -432,12 +432,6 @@ class TranslatorEnglish : public Translator
     virtual QCString trFileDocumentation()
     { return "File Documentation"; }
 
-    /*! This is used in LaTeX as the title of the chapter containing
-     *  the documentation of all examples.
-     */
-    virtual QCString trExampleDocumentation()
-    { return "Example Documentation"; }
-
     /*! This is used in LaTeX as the title of the document */
     virtual QCString trReferenceManual()
     { return "Reference Manual"; }
@@ -2054,7 +2048,7 @@ class TranslatorEnglish : public Translator
     virtual QCString trFunctionAndProc()
     { return "Functions/Procedures/Processes"; }
     /** VHDL type */
-    virtual QCString trVhdlType(uint64 type,bool single)
+    virtual QCString trVhdlType(uint64_t type,bool single)
     {
       switch(type)
       {
@@ -2349,6 +2343,228 @@ class TranslatorEnglish : public Translator
      *  trMemberFunctionDocumentation(), now better adapted for
      *  VHDL sources documentation.
      */
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.9.7
+//////////////////////////////////////////////////////////////////////////
+    /*! used in the compound documentation before a list of related symbols.
+     *
+     *  Supersedes trRelatedFunctions
+     */
+    virtual QCString trRelatedSymbols()
+    { return "Related Symbols"; }
+
+    /*! subscript for the related symbols
+     *
+     *  Supersedes trRelatedSubscript
+     */
+    virtual QCString trRelatedSymbolsSubscript()
+    { return "(Note that these are not member symbols.)"; }
+
+    /*! used in the class documentation as a header before the list of all
+     * related classes.
+     *
+     * Supersedes trRelatedFunctionDocumentation
+     */
+    virtual QCString trRelatedSymbolDocumentation()
+    { return "Friends And Related Symbol Documentation"; }
+
+    /*! the compound type as used for the xrefitems */
+    virtual QCString trCompoundType(ClassDef::CompoundType compType, SrcLangExt lang)
+    {
+      QCString result;
+      switch(compType)
+      {
+        case ClassDef::Class:
+          if (lang == SrcLangExt_Fortran) trType(true,true);
+          else result=trClass(true,true);
+          break;
+        case ClassDef::Struct:     result="Struct"; break;
+        case ClassDef::Union:      result="Union"; break;
+        case ClassDef::Interface:  result="Interface"; break;
+        case ClassDef::Protocol:   result="Protocol"; break;
+        case ClassDef::Category:   result="Category"; break;
+        case ClassDef::Exception:  result="Exception"; break;
+        case ClassDef::Service:    result="Service"; break;
+        case ClassDef::Singleton:  result="Singleton"; break;
+        default: break;
+      }
+      return result;
+    }
+
+    virtual QCString trFileMembersDescriptionTotal(FileMemberHighlight::Enum hl)
+    {
+      bool extractAll = Config_getBool(EXTRACT_ALL);
+      QCString result="Here is a list of all ";
+      if (!extractAll) result+="documented ";
+
+      switch (hl)
+      {
+        case FileMemberHighlight::All:
+          if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
+          {
+            result+="functions, variables, defines, enums, and typedefs";
+          }
+          else
+          {
+            result+="file members";
+          }
+          break;
+        case FileMemberHighlight::Functions:
+          result+="functions";
+          break;
+        case FileMemberHighlight::Variables:
+          result+="variables";
+          break;
+        case FileMemberHighlight::Typedefs:
+          result+="typedefs";
+          break;
+        case FileMemberHighlight::Sequences:
+          result+="sequences";
+          break;
+        case FileMemberHighlight::Dictionaries:
+          result+="dictionaries";
+          break;
+        case FileMemberHighlight::Enums:
+          result+="enums";
+          break;
+        case FileMemberHighlight::EnumValues:
+          result+="enum values";
+          break;
+        case FileMemberHighlight::Defines:
+          result+="macros";
+          break;
+        case FileMemberHighlight::Total: // for completeness
+          break;
+      }
+      result+=" with links to ";
+      if (extractAll)
+        result+="the files they belong to:";
+      else
+        result+="the documentation:";
+      return result;
+    }
+    virtual QCString trCompoundMembersDescriptionTotal(ClassMemberHighlight::Enum hl)
+    {
+      bool extractAll = Config_getBool(EXTRACT_ALL);
+      QCString result="Here is a list of all ";
+      if (!extractAll)
+      {
+        result+="documented ";
+      }
+
+      switch (hl)
+      {
+        case ClassMemberHighlight::All:
+          if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
+          {
+            result+="struct and union fields";
+          }
+          else
+          {
+            result+="class members";
+          }
+          break;
+        case ClassMemberHighlight::Functions:
+          result+="functions";
+          break;
+        case ClassMemberHighlight::Variables:
+          result+="variables";
+          break;
+        case ClassMemberHighlight::Typedefs:
+          result+="typedefs";
+          break;
+        case ClassMemberHighlight::Enums:
+          result+="enums";
+          break;
+        case ClassMemberHighlight::EnumValues:
+          result+="enum values";
+          break;
+        case ClassMemberHighlight::Properties:
+          result+="properties";
+          break;
+        case ClassMemberHighlight::Events:
+          result+="events";
+          break;
+        case ClassMemberHighlight::Related:
+          result+="related symbols";
+          break;
+        case ClassMemberHighlight::Total: // for completeness
+          break;
+      }
+      result+=" with links to ";
+      if (!extractAll)
+      {
+        if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
+        {
+          result+="the struct/union documentation for each field:";
+        }
+        else
+        {
+          result+="the class documentation for each member:";
+        }
+      }
+      else
+      {
+        if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
+        {
+          result+="the structures/unions they belong to:";
+        }
+        else
+        {
+          result+="the classes they belong to:";
+        }
+      }
+      return result;
+    }
+    virtual QCString trNamespaceMembersDescriptionTotal(NamespaceMemberHighlight::Enum hl)
+    {
+      bool extractAll = Config_getBool(EXTRACT_ALL);
+      QCString result="Here is a list of all ";
+      if (!extractAll) result+="documented ";
+      result+="namespace ";
+      QCString singularResult = "";
+      QCString pluralResult = "";
+      switch (hl)
+      {
+        case NamespaceMemberHighlight::All:
+          singularResult="member";
+          break;
+        case NamespaceMemberHighlight::Functions:
+          singularResult="function";
+          break;
+        case NamespaceMemberHighlight::Variables:
+          singularResult="variable";
+          break;
+        case NamespaceMemberHighlight::Typedefs:
+          singularResult="typedef";
+          break;
+        case NamespaceMemberHighlight::Sequences:
+          singularResult="sequence";
+          break;
+        case NamespaceMemberHighlight::Dictionaries:
+          singularResult="dictionary";
+          pluralResult="dictionaries";
+          break;
+        case NamespaceMemberHighlight::Enums:
+          singularResult="enum";
+          break;
+        case NamespaceMemberHighlight::EnumValues:
+          singularResult="enum value";
+          break;
+        case NamespaceMemberHighlight::Total: // for completeness
+          break;
+      }
+      result+=(pluralResult.isEmpty() ? singularResult+"s" : pluralResult);
+      result+=" with links to ";
+      if (extractAll)
+        result+="the namespace documentation for each " + singularResult + ":";
+      else
+        result+="the namespaces they belong to:";
+      return result;
+    }
+    virtual QCString trDefinition()  { return "Definition";}
+    virtual QCString trDeclaration() { return "Declaration";}
 };
 
 #endif
